<?php 
/*
* Page List Plus for c5
*
* @package Block Quoter
* @author Dave Rogers <connect@skybluesofa.com>
* @version 5.7.0.0.0.1
* @copyright Copyright (c) 2014, Dave Rogers
* @license http://www.concrete5.org/help/legal/commercial_add-on_license/ c5 Commercial Add-On License
*/
namespace Concrete\Package\SkybluesofaPageListPlus;

use \Symfony\Component\ClassLoader\MapClassLoader as SymfonyMapClassloader;
use Concrete\Package\SkybluesofaPageListPlus\Src\Installer as Installer;
use Package;
use Route;
use Router;

defined('C5_EXECUTE') or die("Access Denied.");

class Controller extends Package
{

    protected $pkgDescription = "Page Lists with keyword and page attribute filtering as well as built-in list titles";
    protected $pkgName = "Page List+";
    protected $pkgHandle = 'skybluesofa_page_list_plus';
    protected $appVersionRequired = '5.7.3.1';
    protected $pkgVersion = '1.2.12';

    public static $pageListPlusFilters = array(
        'boolean' => 'Boolean',
        'date_time' => 'DateTime',
        'image_file' => 'ImageFile',
        'number' => 'Number',
        'rating' => 'Rating',
        'select' => 'Select',
        'standard' => 'Standard',
        'text' => 'Text',
        'textarea' => 'Textarea',
        'topics' => 'Topics',
        'multi_date' => 'MultiDate',
        'page_selector' => 'PageSelector'
//        'event_category' => 'EventCategory',
  //      'price' => 'Price',
    );

    public function on_start()
    {
        $this->registerRoutes();
    }

    public function install()
    {
        $this->preInstallationSetup();
        $preInstallationErrors = Installer::preInstallationCheck();
        if (!$preInstallationErrors) {
            $pkg = parent::install();
            Installer::install($pkg);
        } else {
            return $preInstallationErrors;
        }
    }

    public function upgrade()
    {
        $this->preInstallationSetup();
        $preInstallationErrors = Installer::preInstallationCheck();
        if (!$preInstallationErrors) {
            $pkg = $this->getByID($this->getPackageID());
            parent::upgrade();
            Installer::upgrade($pkg);
        } else {
            return $preInstallationErrors;
        }
    }

    public function uninstall()
    {
        $this->preInstallationSetup();
        $pkg = $this->getByID($this->getPackageID());
        Installer::uninstall($pkg);
        parent::uninstall();
    }

    private function registerRoutes()
    {
        Route::register(
            Router::route(array('/preview', 'skybluesofa_page_list_plus')),
            '\Concrete\Package\SkybluesofaPageListPlus\Src\Preview::render'
        );
        Route::register(
            Router::route(array('/reload', 'skybluesofa_page_list_plus')),
            '\Concrete\Package\SkybluesofaPageListPlus\Src\Reload::render'
        );
        Route::register(
            '/rss/{identifier}',
            '\Concrete\Package\SkybluesofaPageListPlus\Src\Feed::get',
            'rss',
            array('identifier' => '[A-Za-z0-9_/.]+')
        );
    }

    private function preInstallationSetup() {
        $this->excludeFilesFromAnnotationChecks();
        $this->registerInstaller();

    }

    private function excludeFilesFromAnnotationChecks()
    {
        $driver = $this->getEntityManager()->getConfiguration()->getMetadataDriverImpl();
        $excludePaths = array();
        $excludePaths[] = DIR_PACKAGES . '/' . $this->pkgHandle . '/src/Installer.php';
        $excludePaths[] = DIR_PACKAGES . '/' . $this->pkgHandle . '/src/PageListPlus/Filter/Filter.php';
        $excludePaths[] = DIR_PACKAGES . '/' . $this->pkgHandle . '/src/PageListPlus/Filter/Contract/FilterContract.php';
        $driver->addExcludePaths($excludePaths);
    }

    private function registerInstaller()
    {
        $symfonyLoader = new SymfonyMapClassloader(array(
            NAMESPACE_SEGMENT_VENDOR . '\\Package\\' . camelcase($this->pkgHandle) . '\\Src\\Installer' =>
                DIR_PACKAGES . '/' . $this->pkgHandle . '/' . DIRNAME_CLASSES . '/Installer.php'
        ));
        $symfonyLoader->register();

    }

}
